@@ -70,7 +70,6 @@ impl Executor {
             }
         });
         runnable.schedule();
-        self.rebalance();
         task
     }
 
@@ -109,14 +108,18 @@ struct TlsState {
     counter: UnsafeCell<usize>,
 }
 
+impl Drop for TlsState {
+    fn drop(&mut self) {
+        for runnable in self.inner_sender.drain(..) {
+            self.global_queue.push(runnable);
+        }
+    }
+}
+
 impl TlsState {
     #[inline]
     unsafe fn schedule_local(&mut self, task: Runnable) -> Result<(), Runnable> {
         *self.counter.get() += 1;
-        // // occasionally, we intentionally fail to push tasks to the global queue. this improves fairness.
-        // if *self.counter.get() % 256 == 0 {
-        //     return Err(task);
-        // }
         self.inner_sender.push(task);
         self.local_notifier.set();
         Ok(())
@@ -136,8 +139,12 @@ pub struct Worker {
 
 impl Drop for Worker {
     fn drop(&mut self) {
-        self.stealers.write().unwrap().remove(self.worker_id);
         TLS.with(|v| v.borrow_mut().take());
+        self.stealers.write().unwrap().remove(self.worker_id);
+        while let Some(task) = self.local_queue.pop() {
+            self.global_queue.push(task);
+        }
+        self.global_notifier.notify(usize::MAX);
     }
 }
 
@@ -155,10 +162,8 @@ impl Worker {
             self.set_tls();
             while let Some(task) = self.run_once(is_global) {
                 task.run();
-                if fastrand::u8(0..=u8::MAX) == 0 {
-                    futures_lite::future::yield_now().await;
-                }
             }
+            futures_lite::future::yield_now().await;
             let local = self.local_notifier.wait();
             is_global = async {
                 local.await;
